import torch
from torch.nn import KLDivLoss, Softmax

from base import Metric


class Fidelity(Metric):
    def __init__(name: str, model: torch.nn.Module, mask_type: str):
        super().__init__(name=name, model=model)
        self.authorized_metric = [
            "fidelity_plus",
            "fidelity_minus",
            "fidelity_plus_prob",
            "fidelity_minus_prob",
            "fidelity_plus_model",
            "fidelity_minus_model",
            "fidelity_plus_prob_model",
            "fidelity_minus_prob_model",
            "infidelity_KL",
        ]

        self.metric = self.load_metric(name)
        self.exp_sub = None
        self.exp_sub_c = None
        self.s_exp_sub = None
        self.s_exp_sub_c = None
        self.s_initial_data = None

    def _fidelity_plus(self, exp) -> float:
        if any(
            [
                attr is None
                for attr in [
                    self.exp_sub,
                    self.exp_sub_c,
                    self.s_exp_sub,
                    self.s_exp_sub_c,
                    self.s_initial_data,
                ]
            ]
        ):
            self.score(exp)
        else:
            fid = self.s_initial_data - self.s_exp_sub_c

    def score(self, exp):
        self.exp_sub = exp.get_explanation_subgraph()
        self.exp_sub_c = exp.get_complement_subgraph()
        self.s_exp_sub = self.get_prediction(self.exp_sub)
        self.s_exp_sub_c = self.get_prediction(self.exp_sub_c)
        self.s_initial_data = self.get_prediction(exp)

    def load_metric(name):
        if name in self.authorized_metric:
            if name == "fidelity_plus":

                self.metric = eval("sklearn.metric.{name}")
        else:
            raise ValueError(f"{name} is not supported")

    def compute(self, mask, target: Tensor) -> float:
        if mask.type() == torch.bool and target.type() == torch.bool:
            return self.metric(y_pred=mask, y_true=target)

    def __call__(self, exp: Explanation):
        pass
